/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ZXCfg.h"
#include "ZXMenu.h"
#include "ZXFiles.h"
#include "ZXSpectrum.h"

using namespace std;

ZXMenu::ZXMenu (ZXULA *aZXULA) : _files(0), _config(0), theZXULA(aZXULA) {
  _rootdir = "/";
  
  _find_depth = 0;
  // _rootdir devrait contenir le chemin complet a l'executable
  findFile ("SpectrumDS.nds", "/"); // On cherche l'executable a partir de /

  string snadir = _rootdir + "sna/";
  _files = new ZXFiles (snadir);

  string cfgfile = snadir + "spectrum.cfg";
  _config = new ZXCfg (cfgfile);

  _it_sna = _files->getSna().begin();
}

ZXMenu::~ZXMenu () {
  hide();

  if (_files) {
	delete _files;
	_files = 0;
  }

  if (_config) {
	delete _config;
	_config=0;
  }
  
}

void ZXMenu::show () {
  PA_LoadGifXY(0, I_CANCEL_X, I_CANCEL_Y, (void*)cancel); // Cancel
  //PA_LoadGifXY(0, I_SCREENSHOT_X, I_SCREENSHOT_Y, (void*)screenshot); // Auto/Manuel
  //PA_LoadGifXY(0, I_TOOLS_X, I_TOOLS_Y, (void*)tools); // Auto/Manuel
  PA_LoadGifXY(0, I_SAVE_X, I_SAVE_Y, (void*)save); // Save
  //PA_LoadGifXY(0, I_SOUND_X, I_SOUND_Y, (void*)sound); // Sound

  PA_LoadGifXY(0, I_LEFT_X, I_LEFT_Y, (void*)left); // previous SNA
  PA_LoadGifXY(0, I_RIGHT_X, I_RIGHT_Y, (void*)right); // next SNA
}

void ZXMenu::hide () {
  PA_OutputSimpleText(0, 0, 17,"                                ");
  PA_Clear16bitBg(0);
  
}

int ZXMenu::process (string &name, deque<string> & sequence) {
  list<string> &sna = _files->getSna();
  if (sna.size() == 0) return 1;

  deque<string> seq = _config->getZXCfg(*_it_sna).getSequence ();

  bool r=true;
  bool s=true;
  bool t=true;
  bool snd = theZXULA->getSound();
  bool tls = true;

  //showSequence (seq);
  while (1) {
	// Keys & Pad
	if (Pad.Newpress.Right) { // Suivant
	  if (_it_sna != sna.end()) {
		_it_sna++;
		if (_it_sna == sna.end()) _it_sna=sna.begin();
		r=true;
	  }
	}
	else if (Pad.Newpress.Left) { // Precedent
	  if (_it_sna == sna.begin()) {
		_it_sna = sna.end();
	  }
	  
	  _it_sna--;
	  r=true;
	}
	else if (Pad.Newpress.Select) { // Joystick suivant
	  theZXULA->changeJoystick ();
	}

	// Stylus
	if (Stylus.Newpress) {
	  if (isStylusArea (64, 40, 128, 96)) { // Lancement
		name = "sna/";
		name += *_it_sna;
		if (tls) sequence = seq;
		else sequence.clear();
		// _config->saveZXCfg(); // on sauvegarde les configurations
		return 0; // OK
	  }
	  else if ( isStylusArea(I_LEFT_X, I_LEFT_Y, I_W, I_H)) { // Precedent
		if (_it_sna == sna.begin()) {
		  _it_sna = sna.end();
		}
		
		_it_sna--;
		r=true;
	  }
	  else if ( isStylusArea(I_RIGHT_X, I_RIGHT_Y, I_W, I_H)) { // Suivant
		if (_it_sna != sna.end()) {
		  _it_sna++;
		  if (_it_sna == sna.end()) _it_sna=sna.begin();
		  r=true;
		}
	  }
	  else if ( isStylusArea(I_CANCEL_X, I_CANCEL_Y, I_W, I_H)) { // Abandon
		return 1;
	  }
	  else if (isStylusArea(I_SCREENSHOT_X, I_SCREENSHOT_Y, I_W, I_H)) { // Screenshot
	  }
	  else if (isStylusArea(I_SAVE_X, I_SAVE_Y, I_W, I_H)) { // Save
	  }
	  else if (isStylusArea(I_TOOLS_X, I_TOOLS_Y, I_W, I_H)) { // Auto/Manual config
		t = true;
		if (tls) tls = false;
		else tls = true;
	  }
	  else if (isStylusArea(I_SOUND_X, I_SOUND_Y, I_W, I_H)) { // Sound
		s = true;
		if (snd) {
		  snd = false;
		  PA_StopSound(0);
		}
		else snd = true;
	  }
	}

	if (t) {
	  t=false;
	  if (tls) PA_LoadGifXY(0, I_TOOLS_X, I_TOOLS_Y, (void*)tools); 
	  else PA_LoadGifXY(0, I_TOOLS_X, I_TOOLS_Y, (void*)notools);
	}

	if (s) {
	  s=false;
	  theZXULA->setSound(snd);
	  if (snd) PA_LoadGifXY(0, I_SOUND_X, I_SOUND_Y, (void*)sound); 
	  else PA_LoadGifXY(0, I_SOUND_X, I_SOUND_Y, (void*)nosound);
	}

	if (r) {
	  r=false;
	  drawGifFromSna (0, *_it_sna);
	  theZXULA->setJoystick(_config->getZXCfg(*_it_sna).getJoy ());
	  seq = _config->getZXCfg(*_it_sna).getSequence ();
	  //showSequence (seq);
	}

	PA_WaitForVBL();
  }
}

int ZXMenu::isStylusArea (u8 x, u8 y, u8 w, u8 h) {
  if (Stylus.X < x || Stylus.X > x+w) return 0;
  if (Stylus.Y < y || Stylus.Y > y+h) return 0;
  return 1;
}

void ZXMenu::drawGifFromSna (u8 screen, const string &file) {
  FILE* rawFile;
  string giffile;

  bool res = _files->getGifFile (file, giffile); // On cherche le gif associe au sna

  if ((res == true) && (rawFile = fopen (giffile.c_str(), "r") ) ) {
	  // calcul de la taille
	  u32 rawSize;
	  fseek (rawFile , 0 , SEEK_END);
	  rawSize = ftell (rawFile);
	  rewind (rawFile);
	  char *gif = new char [rawSize];
	  fread (gif,1,rawSize,rawFile);
	  PA_LoadGifXY (screen, 64, 40, gif);
	  delete [] gif;
	  fclose (rawFile);
  }
  else {
	PA_LoadGifXY (screen, 64, 40,(void *)  nogif); // Chargement de l'image par defaut
  }
  int size = file.size();
  if (size != 0) {
	string name;
	u16 x = (32 - size) / 2;
	PA_OutputSimpleText(0, 0, 17,"                                ");
	PA_OutputText(0, x, 17,"%s", file.c_str());
  }
}

void ZXMenu::showSequence (const deque<string> &seq) const {
  deque<string>::const_iterator it;
  string s;
  for (it = seq.begin(); it != seq.end(); it++) {
	s = s + *it + " ";
  }

  if (s.empty()) {
	PA_OutputSimpleText (0, 0, 19, "                        ");
  }
  else {
	PA_OutputText (0, 0, 19, ": %s", s.c_str());
  }
}

bool ZXMenu::findFile (const string &f, string rep) { 
  DIR_ITER* dir = diropen(rep.c_str());
  if (!dir) return false;


  // On limite la recursion pour ne pas boucler sans fin dans desmume
  if (_find_depth >= FIND_MAX_DEPTH) {
	return false;
  }

  list<string> repertoires;

  struct stat st;
  char tmp[256];
  while (dirnext(dir, tmp, &st) == 0) {
	string filename = tmp;
	if (!(st.st_mode & S_IFDIR)) {// st.st_mode & S_IFDIR indicates a directory
	  if (f == filename) {
		_rootdir = rep;
		return true;
	  }
	}
 	else {
	  if (filename == ".") ;
	  else if (filename == "..") ;
	  else {
		repertoires.push_back(filename);
	  }
	}
  }
  dirclose (dir);

  list<string>::iterator it;
  for (it = repertoires.begin(); it != repertoires.end(); it++) {
	string r = rep;
	r = r + *it;
	r = r + "/";
	_find_depth++;

	if (findFile (f, r) == true) { 
	  return true;
	}
	_find_depth--;
  }

  return false;
}

